#include <avr/io.h>
#include <util/delay.h>
#include "uart.h"

// Defincia pinov
#define DIN_PIN     3    // Pripojenie Arduina k pinu DIN(data in -  odosielanie dt z mastera) na MAX7219  (pin 11)
#define CLK_PIN     5    // Pripojenie Arduina k pinu CLK(alebo SCK - uruje synchronizan frekvenciu pre prenos dt) na MAX7219  (pin 13)
#define CS_PIN      2    // Pripojenie Arduina k pinu CS(chip select - sli na vybratie konkrtneho slave zariadenia) na MAX7219   (pin 10)
#define switchL   PD3
#define switchR   PD2

void delay(int delay)      
{
  for (int i=1; i<=delay; i++)
  _delay_ms(1);
}

// Inicializcia SPI komunikcie
void SPI_init() {
    DDRB |= (1 << DIN_PIN) | (1 << CLK_PIN) | (1 << CS_PIN); // Nastavenie pinov DIN, CLK, CS ako vstupy
    PORTB |= (1 << CS_PIN); // Nastavenie CS pinu na HIGH (deaktivovan stav)
    SPCR |= (1 << SPE) | (1 << MSTR) | (1 << SPR0); // Zapnutie SPI, Master mdu a nastavenie hodinovho delia na Fosc/16
}

// Funkcia pre odoslanie dt na MAX7219
void MAX7219_send(unsigned char reg, unsigned char data) {
    PORTB &= ~(1 << CS_PIN); // Aktivcia ipu, nastavenie CS na LOW
    SPDR = reg; // Odoslanie adresy registra
    while (!(SPSR & (1 << SPIF))); // akanie na dokonenie odosielania
    SPDR = data; // Odoslanie dt
    while (!(SPSR & (1 << SPIF))); // akanie na dokonenie odosielania
    PORTB |= (1 << CS_PIN); // Deaktivcia ipu, nastavenie CS na HIGH
}

// Funkcia na inicializciu MAX7219
void MAX7219_init() {
    MAX7219_send(0x09, 0x00); // Nastavenie Decode Mode na "no decode"
    MAX7219_send(0x0a, 0x01); // Intenzita svetla (0x00 - 0x0f)
    MAX7219_send(0x0b, 0x07); // Nastavenie Scan Limit (0x07 = zobrazenie 8 riadkov)
    MAX7219_send(0x0c, 0x01); // Nastavenie Shutdown reimu na "normal operation"
    MAX7219_send(0x0f, 0x00); // Test mode off
}

// Funkcia na zobrazenie sipky do prava
void vpravo() {
    unsigned char sipkaVpravo[] = {
        0x00, 0x10, 0x18, 0xfc, 0xfe, 0xfc, 0x18, 0x10
    };

    for (int repeat = 0; repeat < 10; repeat++) { // Vonkaj cyklus pre opakovanie animcie 10-krt
        for (int posun = 0; posun <= 12; posun++) { // "ako daleko sipka prejde"
            for (int i = 1; i <= 8; i++) {  // Prechod vetkch 8 riadkov
                MAX7219_send(i, sipkaVpravo[i - 1]>>posun);
                _delay_ms(9); // Oneskor pr v kadom kroku, aby sa zobrazil pohyb
				if (bit_is_clear(PIND, switchL)) {
					printf("Smer sipok: <--\r");
					return 0;
				}
            }
        }
    }
}

// Funkcia na zobrazenie sipky do ava
void vlavo() {
    unsigned char sipkaVlavo[] = {
        0x00,0x08,0x18,0x3f,0x7f,0x3f,0x18,0x08
    };
	for (int repeat = 0; repeat < 10; repeat++) { // Vonkaj cyklus pre opakovanie animcie 10-krt
		for (int posun = 0; posun <= 12; posun++) {
			for (int i = 1; i <= 8; i++) {
				MAX7219_send(i, sipkaVlavo[i - 1]<<posun);
				delay(9);
				if (bit_is_clear(PIND, switchR)) {
					printf("Smer sipok: -->\r");  
					return 0;
				}					
			}
		}
	}	
}
// Funkcia na vyistenie displeja
void clearDisp() {
    for (int i = 1; i <= 8; i++) { // Prechod cez vetky riadky displeja
        MAX7219_send(i, 0x00); // Nastavenie vetkch bitov v riadku na 0 (vymazanie)
    }
}

// Funkcia na poiaton animciu
void smileAnimation() {
    unsigned char happy[] = {
        0x3c,0x42,0xa5,0x81,0xa5,0x99,0x42,0x3c
    };
	unsigned char normal[] = {
        0x3c,0x42,0xa5,0x81,0x81,0xbd,0x42,0x3c
    };
	unsigned char sad[] = {
        0x3c,0x42,0xa5,0x81,0x99,0xa5,0x42,0x3c
    };
	
	for (int repeat = 0; repeat < 3; repeat++) { // Vonkaj cyklus pre opakovanie animcie 3-krt
			for (int i = 1; i <= 8; i++) {
				MAX7219_send(i, happy[i - 1]);
				delay(9);
			}
			delay(700);
			for (int i = 1; i <= 8; i++) {
				MAX7219_send(i, normal[i - 1]);
				delay(9);
			}
			delay(700);
			for (int i = 1; i <= 8; i++) {
				MAX7219_send(i, sad[i - 1]);
				delay(9);
			}
			delay(700);
	}
}